# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.modules.company.tests import CompanyTestMixin
from trytond.tests.test_tryton import ModuleTestCase


class ProductCostWarehouseTestCase(CompanyTestMixin, ModuleTestCase):
    'Test Product Cost Warehouse module'
    module = 'product_cost_warehouse'
    extras = ['account_invoice_stock', 'product_cost_history']


del ModuleTestCase
